@@ -1,10 +1,10 @@
 '''
+Functions related to chromatograms and mass tracks.
 Use integers for RT scan numbers and intensities.
 Flexible binning based on ppm accuracy.
 Use mz tol (default 5 pmm) in XIC construction. 
 XICs without neighbors within x ppm are considered specific (i.e. high selectivity). 
 Low selectivity regions will be still inspected to determine the true number of XICs.
-Leave calibration to Correspondence step.
 '''
 import numpy as np
 from scipy.signal import find_peaks 
@@ -15,81 +15,26 @@ from statsmodels.nonparametric.smoothers_lowess import lowess
 
 from .mass_functions import check_close_mzs
 
-
-def sum_dict(dict1, dict2):
-    new = {}
-    for k in dict2:
-        if k in dict1:
-            new[k] = dict1[k] + dict2[k]
-        else:
-            new[k] = dict2[k]
-    return new
-
-def get_thousandth_bins(mzTree, mz_tolerance_ppm=5, min_timepoints=5, min_peak_height=1000):
-    '''
-    Process all LC-MS spectral data into flexible bins by units of 0.001 amu.
-    mz_tolerance_ppm: m/z tolerance in part-per-million. Used to seggregate m/z regsions here.
-    ms_expt: = pymzml.run.Reader(f)
-    min_intensity: minimal intentsity value, needed because some instruments keep 0s 
-    min_timepoints: minimal consecutive scans to be considered real signal.
-    min_peak_height: a bin is not considered if the max intensity < min_peak_height.
-    
-    Return: a list of flexible bins, [ [(mz, scan_num, intensity_int), ...], ... ]
-    '''
-    def __rough_check_consecutive_scans__(datatuples, check_max_len=20, gap_allowed=2, min_timepoints=min_timepoints):
-        # a list of data points in format of (mz_int, scan_num, intensity_int)
-        if len(datatuples) < check_max_len:
-            min_check_val = gap_allowed + min_timepoints -1 # step is 4 in five consecutive values without gap
-            rts = sorted([x[1] for x in datatuples])
-            steps = [rts[ii]-rts[ii-min_timepoints+1] for ii in range(min_timepoints-1, len(rts))]
-            if min(steps) > min_check_val:
-                return False
-            else:
-                return True
-        else:
-            return True
-
-    def __check_min_peak_height__(datatuples, min_peak_height):
-        if max([x[2] for x in datatuples]) < min_peak_height:
-            return False
-        else:
-            return True
-
-    tol_ = 0.000001 * mz_tolerance_ppm
-    ks = sorted([k for k,v in mzTree.items() if len(v) >= min_timepoints]) # ascending order enforced
-    bins_of_bins = []
-    tmp = [ks[0]]
-    for ii in range(1, len(ks)):
-        _delta = ks[ii] - ks[ii-1]
-        # merge adjacent bins if they are next to each other or within ppm tolerance
-        if _delta==1 or _delta < tol_ * ks[ii]:
-            tmp.append(ks[ii])
-        else:
-            bins_of_bins.append(tmp)
-            tmp = [ks[ii]]
-
-    bins_of_bins.append(tmp)
-    good_bins = []
-    for bin in bins_of_bins:
-        datatuples = []
-        for b in bin:
-            datatuples += mzTree[b]
-        # check the presence of min consecutive RT in small traces, to filter out more noises
-        # e.g. in an example datafile, 5958 reduced to 4511 traces
-        if __check_min_peak_height__(datatuples, min_peak_height) and __rough_check_consecutive_scans__(datatuples):
-            good_bins.append(datatuples)
-    
-    # del mzTree
-    return good_bins
-
+# -----------------------------------------------------------------------------
+# mass Tracks
+# -----------------------------------------------------------------------------
 
 def extract_massTracks_(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=1000):
     '''
+    Extract mass tracks from an object of parsed LC-MS data file.
     A mass track is an EIC for full RT range, without separating the mass traces of same m/z. 
-    ms_expt = pymzml.run.Reader(f)
+
+    Input
+    =====
+    ms_expt = pymzml.run.Reader(f), a parsed object of LC-MS data file
+    mz_tolerance_ppm: m/z tolerance in part-per-million. Used to seggregate m/z regsions here.
+    min_intensity: minimal intentsity value, needed because some instruments keep 0s 
+    min_timepoints: minimal consecutive scans to be considered real signal.
+    min_peak_height: a bin is not considered if the max intensity < min_peak_height.
+
     return 
-    rt_numbers, rt_times,
-    tracks as [( mz, np.array(intensities at full rt range) ), ...]
+    ======
+    {rt_numbers, rt_times, tracks as [( mz, np.array(intensities at full rt range) ), ...]}
     '''
     alldata = []
     rt_times = []           # in seconds
@@ -138,90 +83,6 @@ def extract_massTracks_(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_time
         'tracks': updated_tracks,
     }
 
-# -----------------------------------------------------------------------------
-# indexing function
-# -----------------------------------------------------------------------------
-
-def get_thousandth_regions(ms_expt, mz_tolerance_ppm=5, min_intensity=100, min_timepoints=5, min_peak_height=1000):
-    '''
-    Process all LC-MS spectral data into flexible bins by units of 0.001 amu.
-    mz_tolerance_ppm: m/z tolerance in part-per-million. Used to seggregate m/z regsions here.
-    ms_expt: pyopenms MSExperiment instance, loaded with LC-MS data.
-    min_intensity: minimal intentsity value, needed because some instruments keep 0s 
-    min_timepoints: minimal consecutive scans to be considered real signal.
-    min_peak_height: a bin is not considered if the max intensity < min_peak_height.
-    
-    Return: a list of flexible bins, [ [(mz, scan_num, intensity_int), ...], ... ]
-    '''
-    def __rough_check_consecutive_scans__(datatuples, check_max_len=20, gap_allowed=2, min_timepoints=min_timepoints):
-        # a list of data points in format of (mz_int, scan_num, intensity_int)
-        if len(datatuples) < check_max_len:
-            min_check_val = gap_allowed + min_timepoints -1 # step is 4 in five consecutive values without gap
-            rts = sorted([x[1] for x in datatuples])
-            steps = [rts[ii]-rts[ii-min_timepoints+1] for ii in range(min_timepoints-1, len(rts))]
-            if min(steps) > min_check_val:
-                return False
-            else:
-                return True
-        else:
-            return True
-
-    def __check_min_peak_height__(datatuples, min_peak_height):
-        if max([x[2] for x in datatuples]) < min_peak_height:
-            return False
-        else:
-            return True
-
-    tol_ = 0.000001 * mz_tolerance_ppm
-    number_spectra = ms_expt.getNrSpectra()
-
-    alldata = []
-    for ii in range(number_spectra):
-        if ms_expt[ii].getMSLevel() == 1:                               # MS Level 1 only
-            for (mz, intensity) in zip(*ms_expt[ii].get_peaks()):
-                if intensity > min_intensity:
-                    alldata.append((mz, ii, int(intensity)))
-
-    #print("extracted %d valide data points." %len(alldata))
-    mzTree = {}
-    for x in alldata:
-        ii = int(x[0]*1000)
-        if ii in mzTree:
-            mzTree[ii].append(x)
-        else:
-            mzTree[ii] = [x]
-
-    del alldata
-    ks = sorted([k for k,v in mzTree.items() if len(v) >= min_timepoints]) # ascending order enforced
-    bins_of_bins = []
-    tmp = [ks[0]]
-    for ii in range(1, len(ks)):
-        _delta = ks[ii] - ks[ii-1]
-        # merge adjacent bins if they are next to each other or within ppm tolerance
-        if _delta==1 or _delta < tol_ * ks[ii]:
-            tmp.append(ks[ii])
-        else:
-            bins_of_bins.append(tmp)
-            tmp = [ks[ii]]
-
-    bins_of_bins.append(tmp)
-    good_bins = []
-    for bin in bins_of_bins:
-        datatuples = []
-        for b in bin:
-            datatuples += mzTree[b]
-        # check the presence of min consecutive RT in small traces, to filter out more noises
-        # e.g. in an example datafile, 5958 reduced to 4511 traces
-        if __check_min_peak_height__(datatuples, min_peak_height) and __rough_check_consecutive_scans__(datatuples):
-            good_bins.append(datatuples)
-    
-    # del mzTree
-    return good_bins
-
-
-# -----------------------------------------------------------------------------
-# mass Tracks
-# -----------------------------------------------------------------------------
 
 def extract_single_track_fullrt_length(bin, rt_length):
     '''
@@ -237,28 +98,6 @@ def extract_single_track_fullrt_length(bin, rt_length):
     return ( mz, intensity_track )
 
 
-def extract_single_track_old(bin):
-    '''
-    Phased out after v1.4.
-    A mass track is an EIC for full RT range, without separating the mass traces. 
-    input bins in format of [(mz_int, scan_num, intensity_int), ...].
-    return a massTrack as ( mz, rtlist, intensities ).
-    '''
-    mz = np.mean([x[0] for x in bin])
-    # bin.sort(key=itemgetter(1))   # sort by increasing RT (scan number), not needed any more
-    rtlist = [x[1] for x in bin]
-    min_rt, max_rt = min(rtlist), max(rtlist)
-    rtlist = range(min_rt, max_rt+1)    # filling gaps of RT this way if needed, and sorted
-    _d = {}                             # dict to hold rt to intensity mapping
-    for ii in rtlist: 
-        _d[ii] = 0
-    for r in bin:                       # this gets max intensity on the same RT scan
-        _d[r[1]] = max(r[2], _d[r[1]])
-    intensities = [_d[x] for x in rtlist]
-
-    return ( mz, list(rtlist), intensities ) # range object is not desired - use list
-
-
 def bin_to_mass_tracks(bin_data_tuples, rt_length, mz_tolerance_ppm=5):
     '''
     input a flexible bin by units of 0.001 amu, in format of 
@@ -288,25 +127,80 @@ def bin_to_mass_tracks(bin_data_tuples, rt_length, mz_tolerance_ppm=5):
                 tracks.append( extract_single_track_fullrt_length(
                             bin_data_tuples[hist_starts[left]: hist_starts[right]], rt_length) )
             return tracks
+        # possible improvement to make
         else:
             peak = np.argmax(hist)  # if find_peaks fails, get position of max value
             left = max(0, peak-2)
             right = min(peak+3, num_steps)
             return [extract_single_track_fullrt_length(bin_data_tuples[hist_starts[left]: hist_starts[right]], rt_length)]
 
+
 def merge_two_mass_tracks(T1, T2):
     '''
     massTrack as ( mz, np.array(intensities at full rt range) )
     '''
     return ( 0.5 * (T1[0] + T2[0]), T1[1] + T2[1] )
 
-def merge_two_mass_tracks_old(T1, T2):
+
+def get_thousandth_bins(mzTree, mz_tolerance_ppm=5, min_timepoints=5, min_peak_height=1000):
     '''
-    massTrack as ( mz, rtlist, intensities )
+    Bin an mzTree into a list of data bins, to feed to `bin_to_mass_tracks`.
+    These data bins can form a single mass track, or span larger m/z region 
+    if the m/z values cannot be resolved into discrete tracks here.
+
+    Input
+    =====
+    mzTree: indexed data points, {thousandth_mz: [(mz, ii, intensity_int)...], ...}. 
+            (all data points indexed by m/z to thousandth precision, i.e. 0.001 amu).
+    mz_tolerance_ppm: m/z tolerance in part-per-million. Used to seggregate m/z regsions here.
+    min_intensity: minimal intentsity value, needed because some instruments keep 0s 
+    min_timepoints: minimal consecutive scans to be considered real signal.
+    min_peak_height: a bin is not considered if the max intensity < min_peak_height.
+    
+    Return
+    ======
+    a list of flexible bins, [ [(mz, scan_num, intensity_int), ...], ... ]
     '''
-    mz = 0.5 * (T1[0] + T2[0])
-    d_ = sum_dict( dict(zip(T1[1], T1[2])), dict(zip(T2[1], T2[2])) )
-    return (mz, list(d_.keys()), list(d_.values()))
+    def __rough_check_consecutive_scans__(datatuples, check_max_len=20, gap_allowed=2, min_timepoints=min_timepoints):
+        # a list of data points in format of (mz_int, scan_num, intensity_int)
+        _checked = True
+        if len(datatuples) < check_max_len:
+            min_check_val = gap_allowed + min_timepoints -1 # step is 4 in five consecutive values without gap
+            rts = sorted([x[1] for x in datatuples])
+            steps = [rts[ii]-rts[ii-min_timepoints+1] for ii in range(min_timepoints-1, len(rts))]
+            if min(steps) > min_check_val:
+                _checked = False
+        return _checked
+
+    def __check_min_peak_height__(datatuples, min_peak_height):
+        return max([x[2] for x in datatuples]) >= min_peak_height
+
+    tol_ = 0.000001 * mz_tolerance_ppm
+    ks = sorted([k for k,v in mzTree.items() if len(v) >= min_timepoints]) # ascending order enforced
+    bins_of_bins = []
+    tmp = [ks[0]]
+    for ii in range(1, len(ks)):
+        _delta = ks[ii] - ks[ii-1]
+        # merge adjacent bins if they are next to each other or within ppm tolerance
+        if _delta==1 or _delta < tol_ * ks[ii]:
+            tmp.append(ks[ii])
+        else:
+            bins_of_bins.append(tmp)
+            tmp = [ks[ii]]
+
+    bins_of_bins.append(tmp)
+    good_bins = []
+    for bin in bins_of_bins:
+        datatuples = []
+        for b in bin:
+            datatuples += mzTree[b]
+        # check the presence of min consecutive RT in small traces, to filter out more noises
+        # e.g. in an example datafile, 5958 reduced to 4511 traces
+        if __check_min_peak_height__(datatuples, min_peak_height) and __rough_check_consecutive_scans__(datatuples):
+            good_bins.append(datatuples)
+    
+    # del mzTree
+    return good_bins
 
 
 # -----------------------------------------------------------------------------
@@ -353,16 +247,11 @@ def rt_lowess_calibration(good_landmark_peaks, selected_reference_landmark_peaks
     lowess_predicted = __hacked_lowess__(yy, xx, frac= .2, it=1, xvals=sample_rt_numbers)
     interf = interpolate.interp1d(lowess_predicted, sample_rt_numbers, fill_value="extrapolate")
     ref_interpolated = interf( reference_rt_numbers )
-
     lowess_predicted = [int(round(ii)) for ii in lowess_predicted]
-
-    # check inf on int(round(ii))
-
     rt_cal_dict = dict( 
         [(x,y) for x,y in zip(sample_rt_numbers, lowess_predicted) if x!=y and 0<=y<=reference_rt_bound] )
 
     ref_interpolated = [int(round(ii)) for ii in ref_interpolated]
-
     reverse_rt_cal_dict = dict(
         [(x,y) for x,y in zip(reference_rt_numbers, ref_interpolated) if x!=y and 0<=y<=sample_rt_bound] )
         
@@ -380,12 +269,6 @@ def __hacked_lowess__(yy, xx, frac, it, xvals):
     return interf(xvals)
 
 
-def mock_rt_calibration(sample_rt_numbers, reference_rt_numbers):
-    _total = sorted(set(sample_rt_numbers + reference_rt_numbers))
-    rt_cal_dict = reverse_rt_cal_dict = dict(zip( _total, _total ))
-    return rt_cal_dict, reverse_rt_cal_dict
-    
-
 def savitzky_golay_spline(good_landmark_peaks, selected_reference_landmark_peaks, sample_rt_numbers, reference_rt_numbers):
     '''
     Modified Savitzky-Golay filter followed by spline fitting - pls follow format in rt_lowess.
@@ -417,7 +300,6 @@ def remap_intensity_track(intensity_track, new, rt_cal_dict):
     return new
 
 
-
 # -----------------------------------------------------------------------------
 # smoothing function
 # -----------------------------------------------------------------------------
