@@ -7,7 +7,8 @@ from mass2chem.search import *
 # jms-metabolite-services
 from jms.dbStructures import knownCompoundDatabase, ExperimentalEcpdDatabase
 
-from .samples import SimpleSample
+# from .samples import SimpleSample
+from .mass_functions import complete_mass_paired_mapping
 from .constructors import CompositeMap
 from .json_encoder import NpEncoder
 
@@ -23,27 +24,24 @@ if not sys.warnoptions:
     warnings.simplefilter("ignore")
 
 
-class ext_Experiment():
+class ext_Experiment:
     '''
-    Extend metDataModel.core.Experiment with preprocessing methods.
-    This encapsulates a set of LC-MS files using the same method to be processed together.
+    Similar to metDataModel.core.Experiment with preprocessing methods.
+    This encapsulates a set of LC-MS files using the same experimental method 
+    (chromatography and ionization) to be processed together.
+    E.g., data from postive ESI and negative ESI should not in the same ext_Experiment instance.
     '''
     def __init__(self, sample_registry, parameters):
         '''
         This is the overall container for all data in an experiment/project.
-        Samples are sorted by name and assigned unique IDs.
-
-        Input
-        -----
-        list_input_files: list of inputfiles, including directory path, to read
-        dict_meta_data: description of sample types for each file, e.g. 'QC', 'pooled', 'sample'. Not used now.
-        parameters: including 'ionization_mode', 'min_intensity_threshold', 'min_timepoints'. See main.py.
-
+        sample_registry: dictionary of sample and selected data after mass track extraction.
+            The bulk mass track data are not kept in memory unless specified so.
+        parameters: processing parameters passed from main.py.
         '''
         self.sample_registry = sample_registry
         self.valid_sample_ids = self.get_valid_sample_ids()
         self.number_of_samples = len(self.valid_sample_ids)
-        self.number_scans = 0
+        self.number_scans = self.get_max_scan_number(sample_registry)
         self.all_samples = self.all_sample_instances = []
 
         self.parameters = parameters
@@ -54,7 +52,8 @@ class ext_Experiment():
         
     def get_reference_sample_id(self):
         '''
-        get_reference_sample_id as sample of most number_anchor_mz_pairs
+        get_reference_sample_id either by user specification, or
+        as sample of most number_anchor_mz_pairs, limited to first 100 samples to search.
         '''
         if self.parameters['reference']:
             # match file name; k is sm['sample_id']
@@ -62,35 +61,36 @@ class ext_Experiment():
                 if os.path.basename(self.parameters['reference']) == os.path.basename(v['input_file']):
                     return k
         elif self.sample_registry:
-            L = [(v['number_anchor_mz_pairs'], k) for k,v in self.sample_registry.items()]
-            return sorted(L, reverse=True)[0][1]
+            L = [(v['number_anchor_mz_pairs'], v['sample_id']) for v in self.sample_registry.values()]
+            L.sort(reverse=True)
+            ref = self.sample_registry[L[0][1]]
+            print("\n    The reference sample is:\n    ||* %s *||\n" %ref['name'])
+            print("Max reference retention time is %4.2f at scan number %d.\n" %(
+                max(ref['list_retention_time']), ref['max_scan_number']))
+            return ref['sample_id']
         else:
             return None
         
     def get_valid_sample_ids(self):
         return [k for k,v in self.sample_registry.items() if v['status:eic'] == 'passed']
 
+    def get_max_scan_number(self, sample_registry):
+        return max(
+            [sample_registry[k]['max_scan_number'] for k in self.valid_sample_ids]
+        )
+
     def process_all(self):
         '''
+        This is default asari workflow, whereas samples are aligned via LOWESS regression,
+        and peak detection is performed on composite mass tracks.
         '''
         self.CMAP = CompositeMap(self)
         self.CMAP.construct_mass_grid()
-
-
-        if self.parameters['rt_align_on']:
-
-            self.CMAP.align_retention_time()            # add rt_align_method as arg
-            
-            # some samples could fail alignment; can be processed and aligned at the end
-
-            # add 3rd option of RT align
-
-        else:
+        if not self.parameters['rt_align_on']:
             self.CMAP.mock_rentention_alignment()
 
-
+        self.CMAP.build_composite_tracks()
         self.CMAP.global_peak_detection()
-  
 
     def export_all(self):
         '''
@@ -99,18 +99,17 @@ class ext_Experiment():
         If refDB is used, it's better to be used after all samples are processed, 
         because the m/z values of samples are closer to other samples than refDB.
         '''
-        
         self.CMAP.MassGrid.to_csv(
             os.path.join(self.parameters['outdir'], 'export', self.parameters['mass_grid_mapping']) )
-        self.export_feature_tables()
         self.annotate()
+        self.export_feature_tables()
         self.export_log()
 
-
-    def annotate(self):
+    def annotate(self
+                                    ):
         '''
         Reference databases can be pre-loaded.
-
+        Will verify ppm
         '''
         self.load_annotation_db()
         self.db_mass_calibrate()
@@ -119,7 +118,6 @@ class ext_Experiment():
         EED.build_from_list_peaks(self.CMAP.FeatureList)
         EED.extend_empCpd_annotation(self.KCD)
         EED.annotate_singletons(self.KCD)
-
         self.export_peak_annotation(EED.dict_empCpds, self.KCD, 'Feature_annotation')
         
         # also exporting JSON
@@ -180,9 +178,24 @@ class ext_Experiment():
             'list_matches': [('C6H14N_100.112624', 'M[1+]', 2),
             ('C6H13N_99.104799', 'M+H[1+]', 2)]},...
         '''
+        self.selected_unique_features = {}
         s = "[peak]id_number\tmz\trtime\tapex(scan number)\t[EmpCpd]interim_id\t[EmpCpd]ion_relation\tneutral_formula\tneutral_formula_mass\
         \tname_1st_guess\tmatched_DB_shorts\tmatched_DB_records\n"
-        for ii, V in dict_empCpds.items():
+        for interim_id, V in dict_empCpds.items():
+            if len(V['MS1_pseudo_Spectra']) == 1:
+                self.selected_unique_features[V['MS1_pseudo_Spectra'][0]['id_number']] = (
+                        # empCpd id, neutral_formula, ion_relation
+                        interim_id, V['neutral_formula'], 'singleton'
+                )
+            else:
+                all_peaks = sorted([(peak['peak_area'], peak['goodness_fitting'], peak) for peak in V['MS1_pseudo_Spectra']],
+                                    reverse=True)
+                best_peak = all_peaks[0][2]
+                self.selected_unique_features[best_peak['id_number']] = (
+                    # empCpd id, neutral_formula, ion_relation (will change not always anchor)
+                    interim_id, V['neutral_formula'], best_peak.get('ion_relation', '')
+                )
+
             name_1st_guess, matched_DB_shorts, matched_DB_records = '', '', ''
             if 'list_matches' in V:
                 list_matches = V['list_matches']
@@ -206,10 +219,15 @@ class ext_Experiment():
 
     def export_feature_tables(self, outfile='cmap_feature_table.tsv'):
         '''
-        To export two tables, one is full table under `outdir/export/`, the other filtered/preferred table under `outdir`.
+        To export features tables:
+        1) preferred table under `outdir`, after quality filtering by SNR, peak shape and chromatographic selectivity.
+        2) full table under `outdir/export/`
+        3) unique compound table under `outdir/export/`
+        4) dependent on `target` extract option, a targeted_extraction table under `outdir`.
         '''
-        good_samples = [sample.name for sample in self.all_samples if sample.rt_cal_dict]  # verify sample not dropped in rt_cal
-        filtered_FeatureTable = self.CMAP.FeatureTable[good_samples]                       # non zero counts
+        good_samples = [sample.name for sample in self.all_samples] 
+        filtered_FeatureTable = self.CMAP.FeatureTable[good_samples]                       
+        # non zero counts
         count = filtered_FeatureTable[filtered_FeatureTable>1].count(axis='columns')
         self.CMAP.FeatureTable['detection_counts'] = count
 
@@ -225,74 +243,53 @@ class ext_Experiment():
 
         outfile = os.path.join(self.parameters['outdir'], 'export', 'full_'+self.parameters['output_feature_table'])
         filtered_FeatureTable.to_csv(outfile, index=False, sep="\t")
-        print("Feature table (%d x %d) was written to %s." %(
+        print("\nFeature table (%d x %d) was written to %s." %(
                                 filtered_FeatureTable.shape[0], self.number_of_samples, outfile))
 
+        # extract targeted m/z features
+        if 'target' in self.parameters and self.parameters['target']:  
+            matched_list, _, target_unmapped = complete_mass_paired_mapping(
+                filtered_FeatureTable['mz'].to_list(), self.parameters['target'], self.parameters['mz_tolerance_ppm']
+            )
+            print("\nIn targeted extraction, %d target mz values are not found in this dataset: " %len(target_unmapped))
+            print('    ', [self.parameters['target'][ii] for ii in target_unmapped])
+            matched_targets = [self.parameters['target'][ii[1]] for ii in matched_list]
+            targeted_table = filtered_FeatureTable.iloc[[x[0] for x in matched_list], :]
+            targeted_table.insert(0, "query_target", matched_targets)
+            outfile = os.path.join(self.parameters['outdir'], 'targeted_extraction__'+self.parameters['output_feature_table'])
+            targeted_table.to_csv(outfile, index=False, sep="\t")
+            print("Targeted extraction Feature table (%d x %d) was written to %s.\n" %(
+                                targeted_table.shape[0], self.number_of_samples, outfile))
+
         outfile = os.path.join(self.parameters['outdir'], 'preferred_'+self.parameters['output_feature_table'])
         # hard coded cutoff here for now
         filtered_FeatureTable = filtered_FeatureTable[ filtered_FeatureTable['snr']>10][
-                        filtered_FeatureTable['goodness_fitting']>0.7][filtered_FeatureTable['cSelectivity']>0.7 ]
+                                    filtered_FeatureTable['goodness_fitting']>0.7][
+                                    filtered_FeatureTable['cSelectivity']>0.7 ]
         filtered_FeatureTable.to_csv(outfile, index=False, sep="\t")
         print("Filtered Feature table (%d x %d) was written to %s.\n" %(
                                 filtered_FeatureTable.shape[0], self.number_of_samples, outfile))
         
+        # in self.selected_unique_features: (empCpd id, neutral_formula, ion_relation)
+        sel = [ii for ii in filtered_FeatureTable.index if filtered_FeatureTable['id_number'][ii] in
+                                    self.selected_unique_features.keys()]
+        unique_compound_table = filtered_FeatureTable.loc[sel, :]
+        unique_compound_table.insert(3, "empCpd", [self.selected_unique_features[ii][0] for ii in unique_compound_table['id_number']])
+        unique_compound_table.insert(4, "neutral_formula", [self.selected_unique_features[ii][1] for ii in unique_compound_table['id_number']])
+        unique_compound_table.insert(5, "ion_relation", [self.selected_unique_features[ii][2] for ii in unique_compound_table['id_number']])
+        
+        outfile = os.path.join(self.parameters['outdir'], 'export', 'unique_compound__'+self.parameters['output_feature_table'])
+        unique_compound_table.to_csv(outfile, index=False, sep="\t")
+        print("Unique compound table (%d x %d) was written to %s.\n" %(
+                            unique_compound_table.shape[0], self.number_of_samples, outfile))
+
         
     def export_log(self):
         '''
         Export project parameters to project.json.
-        asari viz can look for project.json when launched standalone?
+        Should asari viz can look for project.json when launched standalone?
 
         '''
         outfile = os.path.join(self.parameters['outdir'], 'project.json')
         with open(outfile, 'w', encoding='utf-8') as f:
             json.dump(self.parameters, f, cls=NpEncoder, ensure_ascii=False, indent=2)
-
-
-#---------------------------------------------------------------------------------------------------------------
-# not used for now
-
-    def store_initiation_samples(self, init_Samples):
-        '''Since initiation samples were proecssed in memory,
-        this step store them according to experiment.database_mode
-        '''
-        for sample in init_Samples:
-            if self.database_mode == 'ondisk': 
-                sample.push_to_disk(sample.list_mass_tracks)
-            elif self.database_mode == 'mongo': 
-                sample.push_to_db(sample.list_mass_tracks
-                    # to implement
-                )
-
-
-    def process_single_sample(self, input_file, database_mode):
-        '''
-        Some parameters can be automatically determined here.
-
-        '''
-        mz_tolerance_ppm = self.parameters['mz_tolerance']
-        min_intensity = self.parameters['min_intensity_threshold']
-        min_timepoints = self.parameters['min_timepoints']
-        min_peak_height = self.parameters['min_peak_height']
-        try:
-            SM = SimpleSample(experiment=self, database_mode=database_mode, 
-                                mode=self.mode, input_file=input_file)
-            SM.process( mz_tolerance_ppm, min_intensity, min_timepoints, min_peak_height)
-            return SM
-        except IndexError:
-            print("Input error in sample %s, dropped from processing." %input_file)
-            return None
-        
-
-    def __choose_initiation_samples__(self, N=3):
-        '''
-        N initial samples are chosen to be analyzed first.
-        One best sample among them is chosen as the reference, especially for retention time alignment.
-        '''
-        if self.parameters['initiation_samples']:
-            return self.parameters['initiation_samples']
-        else:
-            if self.number_of_samples < N+1:
-                return self.list_input_files
-            else:
-                return random.sample(self.list_input_files, N)
-
