@@ -1,13 +1,5 @@
 '''
-While classes are provided in metDataModel, 
-mass tracks (i.e. XIC, EIC or chromatogram) and peaks are not instanced as classes in asari
-internal processing for efficiency.
-XICs as [( mz, rtlist, intensities ), ...].
-Peak format: 
-{
-    'id_number': 0, 'mz', 'apex', 'left_base', 'right_base', 'height', 'parent_masstrace_id', 
-    'rtime', 'peak_area', 'goodness_fitting', 'snr', 'cSelectivity',
-}
+Functions for peak detection and evaluation.
 '''
 
 import multiprocessing as mp
@@ -90,10 +82,6 @@ def iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters,
 
 
 
-
-
-
-
 def deep_detect_elution_peaks( mass_track, max_rt_number, 
                 min_peak_height, min_fwhm, min_prominence_threshold,
                 wlen, snr, min_prominence_ratio, iteration,
